"use strict";(self.webpackChunklearninghouse=self.webpackChunklearninghouse||[]).push([[697],{7697:(A,l,o)=>{o.r(l),o.d(l,{BrainsModule:()=>h});var u=o(6895),s=o(9299),c=o(7618),m=o(4377),a=o(8332),e=o(4650),p=o(9066);const T=[{path:"prediction",component:(()=>{class n{get AlertType(){return a.N}}return n.\u0275fac=function(t){return new(t||n)},n.\u0275cmp=e.Xpm({type:n,selectors:[["app-prediction"]],decls:3,vars:4,consts:[[3,"alertType"]],template:function(t,i){1&t&&(e.TgZ(0,"learninghouse-alert",0),e._uU(1),e.ALo(2,"translate"),e.qZA()),2&t&&(e.Q6J("alertType",i.AlertType.error),e.xp6(1),e.hij(" ",e.lcZ(2,2,"common.errors.not_implemented"),"\n"))},dependencies:[a.w,p.X$],encapsulation:2}),n})(),canActivate:[c.a],data:{minimumRole:m.u.USER}},{path:"training",component:(()=>{class n{get AlertType(){return a.N}}return n.\u0275fac=function(t){return new(t||n)},n.\u0275cmp=e.Xpm({type:n,selectors:[["app-training"]],decls:3,vars:4,consts:[[3,"alertType"]],template:function(t,i){1&t&&(e.TgZ(0,"learninghouse-alert",0),e._uU(1),e.ALo(2,"translate"),e.qZA()),2&t&&(e.Q6J("alertType",i.AlertType.error),e.xp6(1),e.hij(" ",e.lcZ(2,2,"common.errors.not_implemented"),"\n"))},dependencies:[a.w,p.X$],encapsulation:2}),n})(),canActivate:[c.a],data:{minimumRole:m.u.TRAINER}}];let y=(()=>{class n{}return n.\u0275fac=function(t){return new(t||n)},n.\u0275mod=e.oAB({type:n}),n.\u0275inj=e.cJS({imports:[s.Bz.forChild(T),s.Bz]}),n})();var f=o(236);let h=(()=>{class n{}return n.\u0275fac=function(t){return new(t||n)},n.\u0275mod=e.oAB({type:n}),n.\u0275inj=e.cJS({imports:[u.ez,f.m,y]}),n})()}}]);