"use strict";(self.webpackChunklearninghouse=self.webpackChunklearninghouse||[]).push([[592],{8151:(a,t,s)=>{s.d(t,{I:()=>u});var n=s(1135),e=s(9646);class u{constructor(){this.state$=new n.X(null)}handleSuccess(){return this.state$.next("success"),(0,e.of)(!0)}handleError(r){return this.state$.next(r.key),(0,e.of)(!1)}get isSuccess(){return"success"===this.state$.getValue()}}}}]);